"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createParameter = void 0;
const ts = require("typescript");
const models_1 = require("../../models");
const convert_expression_1 = require("../convert-expression");
const converter_1 = require("../converter");
/**
 * Create a parameter reflection for the given node.
 *
 * @param context  The context object describing the current state the converter is in.
 * @param node  The parameter node that should be reflected.
 * @returns The newly created parameter reflection.
 */
function createParameter(context, node) {
    if (!(context.scope instanceof models_1.SignatureReflection)) {
        throw new Error("Expected signature reflection.");
    }
    const signature = context.scope;
    if (!node.symbol) {
        return;
    }
    const parameter = new models_1.ParameterReflection(node.symbol.name, models_1.ReflectionKind.Parameter, signature);
    context.registerReflection(parameter);
    context.withScope(parameter, () => {
        var _a;
        parameter.type = context.converter.convertType(context, (_a = node.type) !== null && _a !== void 0 ? _a : context.getTypeAtLocation(node));
        if (ts.isArrayBindingPattern(node.name) ||
            ts.isObjectBindingPattern(node.name)) {
            parameter.name = "__namedParameters";
        }
        parameter.defaultValue = convert_expression_1.convertDefaultValue(node);
        parameter.setFlag(models_1.ReflectionFlag.Optional, !!node.questionToken);
        parameter.setFlag(models_1.ReflectionFlag.Rest, !!node.dotDotDotToken);
        parameter.setFlag(models_1.ReflectionFlag.DefaultValue, !!parameter.defaultValue);
        if (!signature.parameters) {
            signature.parameters = [];
        }
        signature.parameters.push(parameter);
    });
    context.trigger(converter_1.Converter.EVENT_CREATE_PARAMETER, parameter, node);
    return parameter;
}
exports.createParameter = createParameter;
//# sourceMappingURL=parameter.js.map