"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createReferenceOrDeclarationReflection = exports.createReferenceType = void 0;
const models_1 = require("../../models");
const converter_1 = require("../converter");
/**
 * Create a new reference type pointing to the given symbol.
 *
 * @param context  The context object describing the current state the converter is in.
 * @param symbol  The symbol the reference type should point to.
 * @param includeParent  Should the name of the parent be provided within the fallback name?
 * @returns A new reference type instance pointing to the given symbol.
 */
function createReferenceType(context, symbol, includeParent) {
    if (!symbol) {
        return;
    }
    const checker = context.checker;
    let name = checker.symbolToString(symbol);
    if (includeParent && symbol.parent) {
        name = checker.symbolToString(symbol.parent) + "." + name;
    }
    return new models_1.ReferenceType(name, symbol, context.project);
}
exports.createReferenceType = createReferenceType;
function createReferenceOrDeclarationReflection(context, source, target) {
    var _a, _b;
    var _c;
    if (!(context.scope instanceof models_1.ContainerReflection)) {
        throw new Error("Cannot add reference to a non-container");
    }
    // If any declaration is outside, the symbol should be considered outside. Some declarations may
    // be inside due to declaration merging.
    const declarations = target.getDeclarations();
    if (!declarations ||
        declarations.some((d) => context.isOutsideDocumentation(d.getSourceFile().fileName))) {
        return;
    }
    let reflection = undefined;
    if (context.project.getReflectionFromSymbol(target)) {
        reflection = new models_1.ReferenceReflection(source.name, target, context.scope);
        context.registerReflection(reflection, source);
        (_a = (_c = context.scope).children) !== null && _a !== void 0 ? _a : (_c.children = []);
        context.scope.children.push(reflection);
        context.trigger(converter_1.Converter.EVENT_CREATE_DECLARATION, reflection);
    }
    else if ((_b = target.getDeclarations()) === null || _b === void 0 ? void 0 : _b[0]) {
        const refl = context.converter.convertNode(context, target.declarations[0]);
        if (refl instanceof models_1.DeclarationReflection) {
            refl.name = source.name;
            reflection = refl;
        }
    }
    return reflection;
}
exports.createReferenceOrDeclarationReflection = createReferenceOrDeclarationReflection;
//# sourceMappingURL=reference.js.map