"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTypeParameter = void 0;
const index_1 = require("../../models/index");
const converter_1 = require("../converter");
/**
 * Create a type parameter reflection for the given node.
 *
 * @param context  The context object describing the current state the converter is in.
 * @param node  The type parameter node that should be reflected.
 * @returns The newly created type parameter reflection.
 */
function createTypeParameter(context, node) {
    if (!node.symbol) {
        return;
    }
    const typeParameter = new index_1.TypeParameterType(node.symbol.name);
    if (node.constraint) {
        typeParameter.constraint = context.converter.convertType(context, node.constraint);
    }
    if (node.default) {
        typeParameter.default = context.converter.convertType(context, node.default);
    }
    const reflection = context.scope;
    const typeParameterReflection = new index_1.TypeParameterReflection(typeParameter, reflection);
    if (!reflection.typeParameters) {
        reflection.typeParameters = [];
    }
    reflection.typeParameters.push(typeParameterReflection);
    context.registerReflection(typeParameterReflection);
    context.trigger(converter_1.Converter.EVENT_CREATE_TYPE_PARAMETER, typeParameterReflection, node);
    return typeParameter;
}
exports.createTypeParameter = createTypeParameter;
//# sourceMappingURL=type-parameter.js.map