"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockConverter = void 0;
const ts = require("typescript");
const index_1 = require("../../models/index");
const index_2 = require("../factories/index");
const components_1 = require("../components");
const __1 = require("..");
const fs_1 = require("../../utils/fs");
const path_1 = require("path");
let BlockConverter = class BlockConverter extends components_1.ConverterNodeComponent {
    constructor() {
        super(...arguments);
        /**
         * List of supported TypeScript syntax kinds.
         */
        this.supports = [
            ts.SyntaxKind.ModuleBlock,
            ts.SyntaxKind.SourceFile,
        ];
    }
    initialize() {
        super.initialize();
        this.owner.on(__1.Converter.EVENT_BEGIN, () => {
            this.entryPoints = this.application.options
                .getValue("entryPoints")
                .map((path) => this.normalizeFileName(path_1.resolve(path)));
            this.baseDir = fs_1.getCommonDirectory(this.entryPoints);
        });
    }
    /**
     * Analyze the given class declaration node and create a suitable reflection.
     *
     * @param context  The context object describing the current state the converter is in.
     * @param node     The class declaration node that should be analyzed.
     * @return The resulting reflection or NULL.
     */
    convert(context, node) {
        var _a;
        if (node.kind === ts.SyntaxKind.SourceFile) {
            return this.convertSourceFile(context, node);
        }
        else {
            for (const exp of this.getExports(context, node)) {
                for (const decl of (_a = exp.getDeclarations()) !== null && _a !== void 0 ? _a : []) {
                    this.owner.convertNode(context, decl);
                }
            }
        }
        return context.scope;
    }
    /**
     * Analyze the given source file node and create a suitable reflection.
     *
     * @param context  The context object describing the current state the converter is in.
     * @param node     The source file node that should be analyzed.
     * @return The resulting reflection or NULL.
     */
    convertSourceFile(context, node) {
        let result = context.scope;
        context.withSourceFile(node, () => {
            var _a;
            if (this.isEntryPoint(node.fileName)) {
                const symbol = (_a = context.checker.getSymbolAtLocation(node)) !== null && _a !== void 0 ? _a : node.symbol;
                if (context.inFirstPass) {
                    if (this.entryPoints.length === 1) {
                        result = context.project;
                        context.project.registerReflection(result, symbol);
                    }
                    else {
                        result = index_2.createDeclaration(context, node, index_1.ReflectionKind.Module, this.getModuleName(node.fileName));
                    }
                    context.withScope(result, () => {
                        this.convertExports(context, node);
                    });
                }
                else if (symbol) {
                    result = context.project.getReflectionFromSymbol(symbol);
                    context.withScope(result, () => {
                        this.convertReExports(context, node);
                    });
                }
            }
            else {
                result = index_2.createDeclaration(context, node, index_1.ReflectionKind.Module, this.getModuleName(node.fileName));
                context.withScope(result, () => {
                    this.convertExports(context, node);
                });
            }
        });
        return result;
    }
    convertExports(context, node) {
        var _a;
        // We really need to rebuild the converters to work on a symbol basis rather than a node
        // basis... this relies on us getting declaration merging right, which is dangerous at best
        for (const exp of this.getExports(context, node).filter((exp) => { var _a; return (_a = context
            .resolveAliasedSymbol(exp)
            .getDeclarations()) === null || _a === void 0 ? void 0 : _a.every((d) => d.getSourceFile() === node.getSourceFile()); })) {
            for (const decl of (_a = exp.getDeclarations()) !== null && _a !== void 0 ? _a : []) {
                this.owner.convertNode(context, decl);
            }
        }
    }
    convertReExports(context, node) {
        var _a;
        for (const exp of this.getExports(context, node).filter((exp) => { var _a; return (_a = context
            .resolveAliasedSymbol(exp)
            .getDeclarations()) === null || _a === void 0 ? void 0 : _a.some((d) => d.getSourceFile() !== node.getSourceFile()); })) {
            for (const decl of (_a = exp.getDeclarations()) !== null && _a !== void 0 ? _a : []) {
                this.owner.convertNode(context, decl);
            }
        }
    }
    getExports(context, node) {
        var _a;
        let symbol = (_a = context.checker.getSymbolAtLocation(node)) !== null && _a !== void 0 ? _a : node.symbol;
        if (!symbol && ts.isModuleBlock(node)) {
            symbol = context.checker.getSymbolAtLocation(node.parent.name);
        }
        if (symbol) {
            return context.checker.getExportsOfModule(symbol);
        }
        // This is a global file, get all symbols declared in this file...
        // this isn't the best solution, it would be nice to have all globals given to a special
        // "globals" file, but this is uncommon enough that I'm skipping it for now.
        const sourceFile = node.getSourceFile();
        return context.checker
            .getSymbolsInScope(node, ts.SymbolFlags.Type | ts.SymbolFlags.Value)
            .filter((s) => { var _a; return (_a = s
            .getDeclarations()) === null || _a === void 0 ? void 0 : _a.some((d) => d.getSourceFile() === sourceFile); });
    }
    getModuleName(fileName) {
        return this.normalizeFileName(path_1.relative(this.baseDir, fileName)).replace(/(\.d)?\.[tj]sx?$/, "");
    }
    isEntryPoint(fileName) {
        return this.entryPoints.includes(fileName);
    }
    normalizeFileName(fileName) {
        return fileName.replace(/\\/g, "/");
    }
};
BlockConverter = __decorate([
    components_1.Component({ name: "node:block" })
], BlockConverter);
exports.BlockConverter = BlockConverter;
//# sourceMappingURL=block.js.map