"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NamespaceExportConverter = exports.ExportSpecifierConverter = exports.ExportConverter = void 0;
const ts = require("typescript");
const index_1 = require("../../models/index");
const components_1 = require("../components");
const reference_1 = require("../factories/reference");
let ExportConverter = class ExportConverter extends components_1.ConverterNodeComponent {
    constructor() {
        super(...arguments);
        /**
         * List of supported TypeScript syntax kinds.
         */
        this.supports = [ts.SyntaxKind.ExportAssignment];
    }
    convert(context, node) {
        let symbol;
        // default export
        if (node.symbol &&
            (node.symbol.flags & ts.SymbolFlags.Alias) === ts.SymbolFlags.Alias) {
            symbol = context.checker.getAliasedSymbol(node.symbol);
        }
        else {
            const type = context.getTypeAtLocation(node.expression);
            symbol = type ? type.symbol : undefined;
        }
        if (symbol && symbol.declarations) {
            const project = context.project;
            symbol.declarations.forEach((declaration) => {
                if (!declaration.symbol) {
                    return;
                }
                const reflection = project.getReflectionFromSymbol(declaration.symbol);
                if (node.isExportEquals &&
                    reflection instanceof index_1.DeclarationReflection) {
                    reflection.setFlag(index_1.ReflectionFlag.ExportAssignment, true);
                }
            });
        }
        return context.scope;
    }
};
ExportConverter = __decorate([
    components_1.Component({ name: "node:export" })
], ExportConverter);
exports.ExportConverter = ExportConverter;
let ExportSpecifierConverter = class ExportSpecifierConverter extends components_1.ConverterNodeComponent {
    constructor() {
        super(...arguments);
        this.supports = [ts.SyntaxKind.ExportSpecifier];
    }
    convert(context, node) {
        var _a;
        const source = context.expectSymbolAtLocation(node.name);
        const target = context.resolveAliasedSymbol(context.expectSymbolAtLocation((_a = node.propertyName) !== null && _a !== void 0 ? _a : node.name));
        return reference_1.createReferenceOrDeclarationReflection(context, source, target);
    }
};
ExportSpecifierConverter = __decorate([
    components_1.Component({ name: "node:export-specifier" })
], ExportSpecifierConverter);
exports.ExportSpecifierConverter = ExportSpecifierConverter;
let NamespaceExportConverter = class NamespaceExportConverter extends components_1.ConverterNodeComponent {
    constructor() {
        super(...arguments);
        this.supports = [ts.SyntaxKind.NamespaceExport];
    }
    convert(context, node) {
        const source = context.expectSymbolAtLocation(node.name);
        const target = context.resolveAliasedSymbol(source);
        return reference_1.createReferenceOrDeclarationReflection(context, source, target);
    }
};
NamespaceExportConverter = __decorate([
    components_1.Component({ name: "node:namespace-export" })
], NamespaceExportConverter);
exports.NamespaceExportConverter = NamespaceExportConverter;
//# sourceMappingURL=export.js.map