import { ConverterComponent } from "../components";
/**
 * A handler that tries to find the package.json and readme.md files of the
 * current project.
 *
 * The handler traverses the file tree upwards for each file processed by the processor
 * and records the nearest package info files it can find. Within the resolve files, the
 * contents of the found files will be read and appended to the ProjectReflection.
 */
export declare class PackagePlugin extends ConverterComponent {
    readme: string;
    includeVersion: boolean;
    /**
     * The file name of the found readme.md file.
     */
    private readmeFile?;
    /**
     * The file name of the found package.json file.
     */
    private packageFile?;
    /**
     * List of directories the handler already inspected.
     */
    private visited;
    /**
     * Should the readme file be ignored?
     */
    private noReadmeFile?;
    /**
     * Create a new PackageHandler instance.
     */
    initialize(): void;
    /**
     * Triggered when the converter begins converting a project.
     *
     * @param context  The context object describing the current state the converter is in.
     */
    private onBegin;
    /**
     * Triggered when the converter begins converting a source file.
     *
     * @param context  The context object describing the current state the converter is in.
     * @param reflection  The reflection that is currently processed.
     * @param node  The node that is currently processed if available.
     */
    private onBeginDocument;
    /**
     * Triggered when the converter begins resolving a project.
     *
     * @param context  The context object describing the current state the converter is in.
     */
    private onBeginResolve;
}
