"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTypeArgumentsWithDefaults = exports.getTypeParametersOfType = void 0;
const ts = require("typescript");
/**
 * Returns the type parameters of a given type.
 * @param type The type whos type parameters are wanted.
 * @returns The type parameters of the type. An empty array if the type has no type parameters.
 */
function getTypeParametersOfType(type) {
    var _a, _b;
    const declarations = (_b = (_a = type.getSymbol()) === null || _a === void 0 ? void 0 : _a.getDeclarations()) !== null && _b !== void 0 ? _b : [];
    for (const declaration of declarations) {
        if ((ts.isClassDeclaration(declaration) ||
            ts.isInterfaceDeclaration(declaration)) &&
            declaration.typeParameters) {
            return declaration.typeParameters;
        }
    }
    return [];
}
exports.getTypeParametersOfType = getTypeParametersOfType;
/**
 * Returns a list of type arguments. If a type parameter has no corresponding type argument, the default type
 * for that type parameter is used as the type argument.
 * @param typeParams The type parameters for which the type arguments are wanted.
 * @param typeArguments The type arguments as provided in the declaration.
 * @returns The complete list of type arguments with possible default values if type arguments are missing.
 */
function getTypeArgumentsWithDefaults(typeParams, typeArguments) {
    if (!typeArguments || typeParams.length > typeArguments.length) {
        const typeArgumentsWithDefaults = new Array();
        for (let i = 0; i < typeParams.length; ++i) {
            if (typeArguments && typeArguments[i]) {
                typeArgumentsWithDefaults.push(typeArguments[i]);
            }
            else {
                const defaultType = typeParams[i].default;
                if (defaultType) {
                    typeArgumentsWithDefaults.push(defaultType);
                }
            }
        }
        return typeArgumentsWithDefaults;
    }
    return typeArguments;
}
exports.getTypeArgumentsWithDefaults = getTypeArgumentsWithDefaults;
//# sourceMappingURL=types.js.map