"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOptionsHelp = void 0;
const declaration_1 = require("./declaration");
function hasHint(parameter) {
    var _a;
    return (((_a = parameter.type) !== null && _a !== void 0 ? _a : declaration_1.ParameterType.String) === declaration_1.ParameterType.String &&
        "hint" in parameter);
}
/**
 * Prepare parameter information for the [[toString]] method.
 *
 * @param scope  The scope of the parameters whose help should be returned.
 * @returns The columns and lines for the help of the requested parameters.
 */
function getParameterHelp(options) {
    const parameters = options.getDeclarations();
    parameters.sort((a, b) => a.name.localeCompare(b.name, undefined, { sensitivity: "base" }));
    const names = [];
    const helps = [];
    let margin = 0;
    for (const parameter of parameters) {
        if (!parameter.help) {
            continue;
        }
        let name = " --" + parameter.name;
        if (hasHint(parameter)) {
            name += " " + declaration_1.ParameterHint[parameter.hint].toUpperCase();
        }
        names.push(name);
        helps.push(parameter.help);
        margin = Math.max(name.length, margin);
    }
    return { names, helps, margin };
}
function getOptionsHelp(options) {
    const output = ["Usage:", "  typedoc path/to/entry.ts", "", "Options:"];
    const columns = getParameterHelp(options);
    for (let i = 0; i < columns.names.length; i++) {
        const usage = columns.names[i];
        const description = columns.helps[i];
        output.push(usage.padEnd(columns.margin + 2) + description);
    }
    output.push("");
    return output.join("\n");
}
exports.getOptionsHelp = getOptionsHelp;
//# sourceMappingURL=help.js.map